@@ -8,11 +8,11 @@ try:
 except ImportError as e:
     warnings.warn(ImportWarning(e))
 
-VERSION = (0, 7, 7)
+VERSION = (0, 7, 8)
 
 __version__ = ".".join([str(i) for i in VERSION])
 __author__ = "nielstron"
 __author_email__ = "n.muendler@web.de"
-__copyright__ = "Copyright (C) 2019 nielstron"
+__copyright__ = "Copyright (C) 2023 nielstron"
 __license__ = "MIT"
-__url__ = "https://github.com/imperatorlang/uplc"
+__url__ = "https://github.com/imperatorlang/eopsin"
