@@ -65,7 +65,8 @@ BoolOpMap = {
 }
 
 UnaryOpMap = {
-    Not: plt.Not,
+    Not: {BoolInstanceType: plt.Not},
+    USub: {IntegerInstanceType: lambda x: plt.SubtractInteger(plt.Integer(0), x)},
 }
 
 ConstantMap = {
@@ -104,7 +105,18 @@ def extend_statemonad(
     values: typing.List[plt.AST],
     old_statemonad: plt.FunctionalMap,
 ):
-    return plt.FunctionalMapExtend(old_statemonad, [n for n in names], values)
+    """Ensures that the argument is fully evaluated before being passed into the monad (like in imperative languages)"""
+    assert len(names) == len(values), "Unequal amount of names and values passed in"
+    lam_names = [f"a{i}" for i, _ in enumerate(names)]
+    return plt.Apply(
+        plt.Lambda(
+            lam_names,
+            plt.FunctionalMapExtend(
+                old_statemonad, names, [plt.Var(n) for n in lam_names]
+            ),
+        ),
+        *values,
+    )
 
 
 INITIAL_STATE = plt.FunctionalMap()
@@ -161,7 +173,12 @@ class UPLCCompiler(CompilingNodeTransformer):
         )
 
     def visit_UnaryOp(self, node: TypedUnaryOp) -> plt.AST:
-        op = UnaryOpMap.get(type(node.op))
+        opmap = UnaryOpMap.get(type(node.op))
+        assert opmap is not None, f"Operator {type(node.op)} is not supported"
+        op = opmap.get(node.operand.typ)
+        assert (
+            op is not None
+        ), f"Operator {type(node.op)} is not supported for type {node.operand.typ}"
         return plt.Lambda(
             [STATEMONAD],
             op(plt.Apply(self.visit(node.operand), plt.Var(STATEMONAD))),
@@ -278,12 +295,13 @@ class UPLCCompiler(CompilingNodeTransformer):
             return self.visit_sequence([])
         compiled_e = self.visit(node.value)
         # (\{STATEMONAD} -> (\x -> if (x ==b {self.visit(node.targets[0])}) then ({compiled_e} {STATEMONAD}) else ({STATEMONAD} x)))
+        varname = node.targets[0].id
         return plt.Lambda(
             [STATEMONAD],
-            plt.FunctionalMapExtend(
-                plt.Var(STATEMONAD),
-                [node.targets[0].id],
+            extend_statemonad(
+                [varname],
                 [plt.Apply(compiled_e, plt.Var(STATEMONAD))],
+                plt.Var(STATEMONAD),
             ),
         )
 
@@ -299,14 +317,14 @@ class UPLCCompiler(CompilingNodeTransformer):
         # (\{STATEMONAD} -> (\x -> if (x ==b {self.visit(node.targets[0])}) then ({compiled_e} {STATEMONAD}) else ({STATEMONAD} x)))
         return plt.Lambda(
             [STATEMONAD],
-            plt.FunctionalMapExtend(
-                plt.Var(STATEMONAD),
+            extend_statemonad(
                 [node.target.id],
                 [
                     transform_ext_params_map(node.target.typ)(
                         plt.Apply(compiled_e, plt.Var(STATEMONAD))
                     )
                 ],
+                plt.Var(STATEMONAD),
             ),
         )
 
@@ -374,16 +392,15 @@ class UPLCCompiler(CompilingNodeTransformer):
             body.append(tr)
         compiled_body = self.visit_sequence(body[:-1])
         compiled_return = self.visit(body[-1].value)
-        args_state = plt.FunctionalMapExtend(
-            plt.Var(STATEMONAD),
+        args_state = extend_statemonad(
             # the function can see its argument under the argument names
             [a.arg for a in node.args.args],
             [plt.Var(f"p{i}") for i in range(len(node.args.args))],
+            plt.Var(STATEMONAD),
         )
         return plt.Lambda(
             [STATEMONAD],
-            plt.FunctionalMapExtend(
-                plt.Var(STATEMONAD),
+            extend_statemonad(
                 [node.name],
                 [
                     plt.Lambda(
@@ -398,6 +415,7 @@ class UPLCCompiler(CompilingNodeTransformer):
                         ),
                     )
                 ],
+                plt.Var(STATEMONAD),
             ),
         )
 
@@ -453,10 +471,10 @@ class UPLCCompiler(CompilingNodeTransformer):
                         [STATEMONAD, "e"],
                         plt.Apply(
                             self.visit_sequence(node.body),
-                            plt.FunctionalMapExtend(
-                                plt.Var(STATEMONAD),
+                            extend_statemonad(
                                 [node.target.id],
                                 [plt.Var("e")],
+                                plt.Var(STATEMONAD),
                             ),
                         ),
                     ),
@@ -555,10 +573,10 @@ class UPLCCompiler(CompilingNodeTransformer):
     def visit_ClassDef(self, node: TypedClassDef) -> plt.AST:
         return plt.Lambda(
             [STATEMONAD],
-            plt.FunctionalMapExtend(
-                plt.Var(STATEMONAD),
+            extend_statemonad(
                 [node.name],
                 [node.class_typ.constr()],
+                plt.Var(STATEMONAD),
             ),
         )
 
