@@ -5,6 +5,7 @@ from enum import Enum, auto
 from .typed_ast import *
 
 import pluthon as plt
+import uplc.ast as uplc
 
 
 class PythonBuiltIn(Enum):
@@ -92,3 +93,17 @@ class CompilingNodeVisitor(TypedNodeVisitor):
             if isinstance(e, CompilerError):
                 raise e
             raise CompilerError(e, node, self.step)
+
+
+def data_from_json(j: typing.Dict[str, typing.Any]) -> uplc.PlutusData:
+    if "bytes" in j:
+        return uplc.PlutusByteString(bytes.fromhex(j["bytes"]))
+    if "int" in j:
+        return uplc.PlutusInteger(int(j["int"]))
+    if "list" in j:
+        return uplc.PlutusList(list(map(data_from_json, j["list"])))
+    if "map" in j:
+        return uplc.PlutusMap({d["k"]: d["v"] for d in j["map"]})
+    if "constructor" in j and "fields" in j:
+        return uplc.PlutusConstr(j["constructor"], j["fields"])
+    raise NotImplementedError(f"Unknown datum representation {j}")
